// mxuart.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include "mxsp.h"



#ifdef __cplusplus    // If used by C++ code, 
extern "C" {          // we need to export the C interface
#endif

#define MAX_SERIAL_PORT_NUMBER			3
#define MAX_SERIAL_INTERFACE_NUMBER		4

/*
	Initialize Uart
	
	param:
		port  :	 port number	
				
	Return value:
		unsigned int :			return UART handle
*/
HANDLE mxsp_open()
{
	// Do nothing, for function compatibility only
	
	return  0;
}


/*
	close UART	
	param:
		(in)unsigned int fd  :	Handle of the open port		
				
	Return value:
		none
*/
void mxsp_close()
{
	// Do nothing, for function compatibility only

	return;
}






/*
	Get UART Mode
	
	param:
		(in)int index :	Port index (0,1)		
				
	Return value:
		int:		0 : RS232 Mode
					1 : RS485-2W Mode
					2 : RS422 Mode
					3 : RS485-4W Mode
*/
int mxsp_get_interface(HANDLE fd, int port_index)
{
	int		serialInterfaceIndex = 0;

	serialInterfaceIndex = UartModeGet(port_index);

	return serialInterfaceIndex;
}

int UartModeGet(int portNum)
{
	int		serialInterfaceIndex = 0;

	int uartModeBit[2][3] = {{UART_MODE_BIT_COM1_RS232, UART_MODE_BIT_COM1_RS485, UART_MODE_BIT_COM1_RS422},
							 {UART_MODE_BIT_COM2_RS232, UART_MODE_BIT_COM2_RS485, UART_MODE_BIT_COM2_RS422}};

	UCHAR	ucUartMode = 0x0;
	int		portMode = 0x0; 
	BOOL	bResult = TRUE;

	//
	// Check current mode
	//
	ucUartMode = ReadPortData(UART_MODE_IO_ADDRESS);
	//printf("ucUartMode = 0x%x\n", ucUartMode);

	/* Check port mode with register */
	for(int i = 0; i< 3; i++)
	{
		portMode = ucUartMode & (1<<uartModeBit[portNum-3][i]); //portNum starts from 3(COM3, COM4)
			
		if(portMode >= 1)
		{
			portMode = i;
			break;
		}
	}
	serialInterfaceIndex = portMode;
	
	return serialInterfaceIndex;
}

/*
	Set UART Mode
	
	param:
		(in)int index  :	Port index		
		(in)int mode   :	Port mode		
				
	Return value:
		int:			Current Mode.
						0 : RS-232
						1 : RS485-2W
						2 : RS422
						3 : RS485-4W

*/
int mxsp_set_interface(HANDLE fd, int port_index, int port_interface)
{
	int ret=0;

	UartModeSet(port_index, port_interface);


	return ret;
}

int UartModeSet(int portNum, int portMode)
{
	int uartModeBit[2][3] = {{UART_MODE_BIT_COM1_RS232, UART_MODE_BIT_COM1_RS485, UART_MODE_BIT_COM1_RS422},
							 {UART_MODE_BIT_COM2_RS232, UART_MODE_BIT_COM2_RS485, UART_MODE_BIT_COM2_RS422}};

	UCHAR ucCurrentUartMode = 0x0;
	UCHAR ucNextUartMode = 0x0;
	BOOL bResult = TRUE;
	int		serialInterfaceIndex1 = 0;
	int		serialInterfaceIndex2 = 0;

	//
	// Get current mode
	//
	/*ucCurrentUartMode = ReadPortData(UART_MODE_IO_ADDRESS);
	printf("ucCurrentUartMode = 0x%x\n", ucCurrentUartMode);*/

	/* Get current mode of port 1 */
	switch(portNum)
	{
	case 3:
		serialInterfaceIndex1 = UartModeGet(3);
		serialInterfaceIndex2 = UartModeGet(4);
		ucNextUartMode |= (1 << uartModeBit[portNum-3][portMode]); //portNum starts from 3(COM3, COM4)
		ucNextUartMode |= (1 << uartModeBit[1][serialInterfaceIndex2]);
		break;
	case 4:
		serialInterfaceIndex1 = UartModeGet(3);
		serialInterfaceIndex2 = UartModeGet(4);
		ucNextUartMode |= (1 << uartModeBit[portNum-3][portMode]);//portNum starts from 3(COM3, COM4)
		ucNextUartMode |= (1 << uartModeBit[0][serialInterfaceIndex1]);
		break;
	}

	//
	// Set next uart mode
	//
	/* Set next uart value to hardware register */
	WritePortData(UART_MODE_IO_ADDRESS, ucNextUartMode);

	/* Set auto direction control */
	if(portMode == 0) //We are in 232 mode
	{
		DisableAutoDirectionControl(portNum);
	}
	else //We are in 485 mode
	{
		EnableAutoDirectionControl(portNum);
	}
	
	return 0;
}

int SetRegistryValue(DWORD nPort,DWORD nMode)
{
	HKEY	hKey;
	DWORD	ulRet;
	WCHAR	sPort[20];
	DWORD	dwType;
	ULONG	ulSize;
	
	dwType=REG_DWORD;
	ulSize=sizeof(DWORD);
	ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\MOXA\\COM"), NULL, KEY_ALL_ACCESS, &hKey);
	if ( ulRet == ERROR_SUCCESS )
	{
		
		memset(sPort,0,sizeof(sPort));
	
		wsprintf( sPort, L"COM%d", nPort);

		ulRet = RegSetValueEx( hKey, sPort, NULL, dwType, (LPBYTE)&nMode, ulSize );
		if ( ulRet == ERROR_SUCCESS ) 
		{
			//printf("Set COM%d: Mode=%d\n",nPort,nMode);
		}
		else
		{
			printf("Registry value of COM%d does not exist!\n",nPort);
		}
	}
	else
	{
		printf("RegopenKeyEx Fail!\n");
	}
	RegCloseKey(hKey);


	return 0;
}

int GetRegistryValue(DWORD nPort,DWORD* nMode)
{
	HKEY	hKey;
	DWORD	ulRet;
	WCHAR	sPort[20];
	DWORD	dwType;
	ULONG	ulSize;
	
	dwType=REG_DWORD;
	ulSize=sizeof(DWORD);
	ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\MOXA\\COM"), NULL, KEY_ALL_ACCESS, &hKey);
	if ( ulRet == ERROR_SUCCESS )
	{
		
		memset(sPort,0,sizeof(sPort));
	
		wsprintf( sPort, L"COM%d", nPort);
		ulRet = RegQueryValueEx( hKey, sPort, NULL, &dwType, (LPBYTE)nMode, (LPDWORD)&ulSize );
		if ( ulRet == ERROR_SUCCESS ) 
		{
			//printf("Get COM%d: Mode=%d\n",nPort,*nMode);
		}
		else
		{
			printf("Registry value of COM%d does not exist!\n",nPort);
		}
	}
	else
	{
		printf("RegopenKeyEx Fail!\n");
	}

	RegCloseKey(hKey);
	return 0;
}


UCHAR ReadSuperIoChar(ULONG address, int portNum)
{
	BOOL	bResult;
	WORD	wPortAddr = 0x301;
	DWORD	dwPortVal = 0;
	UCHAR	ucMemData = 0;
	BYTE  logicDeivceData = portNum + 5; //Logic device 8 for COM3, Logic device 9 for COM4


	SUPERIOPARAM superIoParam;
	UCHAR ucUartMode  = 0x0;
	UCHAR ucSuperIoData = 0x0;

	//
	// Initialize the super io parameters
	//
	superIoParam.indexPort = 0x2E;
	superIoParam.dataPort = 0x2F;
	superIoParam.enterConfigValue1 = 0x87;
	superIoParam.enterConfigValue2 = 0x01;
	superIoParam.enterConfigValue3 = 0x55;
	superIoParam.exitConfigValue = 0x2;//0xAA;
	superIoParam.logicDeviceIndex = 0x7;
	superIoParam.logicDeviceData = logicDeivceData; //Logic device 8 for COM3, Logic device 9 for COM4
	superIoParam.registerAddress = (BYTE)address;
	superIoParam.cmdDelay = DELAY_1MS;

	//
	// Open driver handle
	//
	bResult = MxGeneralIoOpen();
	if(bResult == FALSE)
	{
		printf("MxGeneralIoOpen fail\n");
	}

	//
	//	Read memory data
	//
	ucMemData = MxGeneralIoReadSuperIoChar(SUPERIO_VENDER_ITE, superIoParam); //Read UCHAR data

	//
	// Shut down WinIo
	//
	MxGeneralIoClose();

	return ucMemData;
}

int WriteSuperIoChar(ULONG	address,int portNum, UCHAR	ucMemData)
{
	BOOL	bResult;
	DWORD	dwPortVal = 0;


	SUPERIOPARAM superIoParam;
	UCHAR ucUartMode  = 0x0;
	UCHAR ucSuperIoData = 0x0;
	BYTE  logicDeivceData = portNum + 5; //Logic device 8 for COM3, Logic device 9 for COM4

	//
	// Initialize the super io parameters
	//
	superIoParam.indexPort = 0x2E;
	superIoParam.dataPort = 0x2F;
	superIoParam.enterConfigValue1 = 0x87;
	superIoParam.enterConfigValue2 = 0x01;
	superIoParam.enterConfigValue3 = 0x55;
	superIoParam.exitConfigValue = 0x2;//0xAA;
	superIoParam.logicDeviceIndex = 0x7;
	superIoParam.logicDeviceData = logicDeivceData;  //logicDeivceData=8 for COM3, =9 for COM4
	superIoParam.registerAddress = (BYTE)address;
	superIoParam.cmdDelay = DELAY_1MS;

	//
	// Open MxGeneralIo
	//
	bResult = MxGeneralIoOpen();
	if(bResult == FALSE)
	{
		printf("MxGeneralIoOpen fail\n");
	}

	//
	//	Write memory data
	//
	//MxGeneralIoWritePortChar(address, ucMemData); //Write UCHAR data
	MxGeneralIoWriteSuperIoChar(SUPERIO_VENDER_ITE, superIoParam, ucMemData);
		
	//
	// Shut down MxGeneralIo
	//
	MxGeneralIoClose();

	return TRUE;
}

UCHAR ReadPortData(ULONG address)
{
	BOOL	bResult;
	WORD	wPortAddr = 0x301;
	DWORD	dwPortVal = 0;
	UCHAR	ucMemData = 0;

	//
	// Open driver handle
	//
	bResult = MxGeneralIoOpen();
	if(bResult == FALSE)
	{
		printf("MxGeneralIoOpen fail\n");
	}

	//
	//	Read memory data
	//
	ucMemData = MxGeneralIoReadPortChar(address); //Read UCHAR data

	//
	// Shut down WinIo
	//
	MxGeneralIoClose();

	return ucMemData;
}

int WritePortData(ULONG	address, UCHAR	ucMemData)
{
	BOOL	bResult;
	DWORD	dwPortVal = 0;

	//
	// Open MxGeneralIo
	//
	bResult = MxGeneralIoOpen();
	if(bResult == FALSE)
	{
		printf("MxGeneralIoOpen fail\n");
	}

	//
	//	Write memory data
	//
	MxGeneralIoWritePortChar(address, ucMemData); //Write UCHAR data
		
	//
	// Shut down MxGeneralIo
	//
	MxGeneralIoClose();

	return TRUE;
}

int EnableAutoDirectionControl(int portNum)
{
	UCHAR ucSuperIoData = 0x0;
	UCHAR ucUartMode = 0x0;
	UCHAR ucAutoDirectionControl = 0x0;
	BOOL bResult = TRUE;

	/* Get current register data from hardware*/
	ucSuperIoData = ReadSuperIoChar(UART_AUTODIRECTION_CONTROL_ADDRESS, portNum); //portNum=3 for COM3, =4 for COM4
	//printf("ucSuperIoData = 0x%x\n", ucSuperIoData);

	/* Enable auto direction control */
	ucAutoDirectionControl = ucSuperIoData | 0x80;
	WriteSuperIoChar(UART_AUTODIRECTION_CONTROL_ADDRESS ,portNum, ucAutoDirectionControl);

	return 0;
}

int DisableAutoDirectionControl(int portNum)
{
	UCHAR ucSuperIoData = 0x0;
	UCHAR ucUartMode = 0x0;
	UCHAR ucAutoDirectionControl = 0x0;
	BOOL bResult = TRUE;

	//
	// Prepare register data for auto direction control
	//

	/* Get current register data from hardware*/
	ucSuperIoData = ReadSuperIoChar(UART_AUTODIRECTION_CONTROL_ADDRESS, portNum); //portNum=3 for COM3, =4 for COM4
	//printf("ucSuperIoData = 0x%x\n", ucSuperIoData);

	/* Disable auto direction control */
	ucAutoDirectionControl = ucSuperIoData & ~0x80;
	WriteSuperIoChar(UART_AUTODIRECTION_CONTROL_ADDRESS, portNum, ucAutoDirectionControl);
	
	return 0;
}

#ifdef __cplusplus    // If used by C++ code, 
}
#endif
